var path    = require( 'path' )
  , async   = require( 'async' )
  , fs      = require( 'fs' )
  , tcase   = require( 't-case' )
  , es      = require( 'event-stream' )
  , paths   = require( path.join( __dirname, 'paths' ) );

/**
 * Transforms the file from a template to the actual module
 *
 * @param  {String}   from Path to the file to convert from
 * @param  {String}   to   Path to convert the file into
 * @param  {String}   name Name of the module
 * @param  {Function} fn   Callback functions
 * @return {Function}      Returns fn
 * @api public
 */

var transformFile = exports.transformFile = function ( from, to, name, fn ) {
  var file  = fs.createWriteStream( to );
  var read  = fs.createReadStream( from, { encoding: 'utf-8' } )
    .pipe( es.replace( '{{Template}}',        tcase.classCase( name ) ) )
    .pipe( es.replace( '{{TemplateName}}',    tcase.classCase( name ) ) )
    .pipe( es.replace( '{{template-name}}',   tcase.paramCase( name ) ) )
    .pipe( es.replace( '{{_template_}}',      tcase.snakeCase( name ) ) )
    .pipe( es.replace( '{{template_name}}',   tcase.snakeCase( name ) ) )
    .pipe( file );

  read.on( 'error', fn );

  file
    .on( 'error', fn )
    .on( 'close', function ( ) {
      fn( null,  to );
    } );
}

/**
 * Compiles a list of files to transform
 *
 * @param  {String[]}   dirs    Path to directories to look into
 * @param  {String[]}   names   Module names
 * @param  {String[]}   tmpls   Which templates we want from the seed
 * @param  {Object}     project Object returned from util.locations.get( )
 * @param  {Function}   fn      Callback functions
 * @return {Functions}          Returns fn
 * @api public
 */

exports.template = function ( dirs, names, tmpls, project, fn ) {
  var files = [ ];

  async.each( names, function ( name, next ) {
    async.each( tmpls, function ( tmpl, _next ) {
      var filter = path.join( paths.getModulePath( project, name ), tmpl );

      async.eachSeries( dirs.filter( function ( dir ) {
        return filter.indexOf( dir ) > -1;
      } ), function ( dir, __next ) {
        var _files        = [ ]
          , templateHome  = paths.getTemplatePath( project, tmpl )
          , finder        = require( 'findit' )( templateHome );

        finder.on( 'file', function ( file ) {
          _files.push( file );
        } );

        finder.on( 'end', function ( ) {
          async.eachSeries( _files, function ( file, ___next ) {
            var _fileName = name.split( path.sep ).splice( -1 )[ 0 ];

            if (_fileName[ _fileName.length-1 ] === "s") {
              _fileName = _fileName.substr( 0, _fileName.length-1 );
            }

            var fileName  = file.replace( templateHome, '' )
                                .replace( 'Template', tcase.snakeCase( _fileName ) )
                                .replace( 'template', tcase.classCase( _fileName ) )
              , to        = path.resolve( path.join( paths.getModulePath( project, name ) , tmpl, fileName  ) );

            transformFile( file, to, name, function ( err, filePath) {
              if (!!err) {
                return ___next( err );
              }

              files.push( filePath );

              ___next( );
            } );
          }, function ( err ) {
            __next( err );
          } );
        } );
      }, _next );
    }, next );
  },
  function ( err ) {
    fn( err, files );
  } );
}
